﻿#include "stdafx.h"
#include "..\..\DuiLib\UIlib.h"
#include "OrganizationTree.h"
#include "ProfileWnd.h"
#include "main_frame.hpp"
#include "NeMessengerDlg.h"
#include "SearchWnd.h"

namespace DuiLib
{

const int kOrganizationItemFileHeight = 24;
const int kOrganizationItemUserHeight = 29;


COrganizationTreeUI::COrganizationTreeUI(CPaintManagerUI& paint_manager, HWND hWnd)
	: root_node_(NULL)
	, delay_deltaY_(0)
	, delay_number_(0)
	, delay_left_(0)
	, level_expand_image_(_T("<i img_main_organization_folder_icon_open.png>"))
	, level_collapse_image_(_T("<i img_main_organization_folder_icon_close.png>"))
	, text_padding_(6, 0, 0, 0)
	, paint_manager_(paint_manager)
{
	SetItemShowHtml(true);

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
	m_hParent = hWnd;
	m_nShiftFirstSel = -1;
	m_nShiftSecondSel = -1;
}

COrganizationTreeUI::~COrganizationTreeUI()
{
	if (root_node_)
		delete root_node_;

	root_node_ = NULL;
}

HWND COrganizationTreeUI::GetParentHwnd()
{
	return m_hParent;
}

std::vector<Node*> COrganizationTreeUI::GetSelectedItems()
{
	vector<Node*> listRet;
	list<int> listSelectedIndexs = CListUI::GetSelectedItems();
	for(list<int>::iterator it = listSelectedIndexs.begin();
		it != listSelectedIndexs.end(); it++)
	{
		CControlUI* pControl = GetItemAt(*it);
		//Node* node = NULL;
		if( pControl != NULL) 
		{
			Node* node = (Node*)pControl->GetTag();
			if(node != NULL)
			{
				listRet.push_back(node);
			}
		}
	}

	return listRet;

/*
	list<int> listSelRows = pOrgTree->GetSelectedItems();
	if (listSelRows.size() > 0)
	{

		list<int>::iterator it = listSelRows.begin();
		for (; it != listSelRows.end(); ++it)
		{
			vecSelID.push_back(pOrgTree->GetItemNode(*it));
		}
	}*/
}

bool COrganizationTreeUI::Add(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::Add(pControl);
}

bool COrganizationTreeUI::AddAt(CControlUI* pControl, int iIndex)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	return CListUI::AddAt(pControl, iIndex);
}

bool COrganizationTreeUI::Remove(CControlUI* pControl)
{
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::Remove(pControl);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

bool COrganizationTreeUI::RemoveAt(int iIndex)
{
	CControlUI* pControl = GetItemAt(iIndex);
	if (!pControl)
		return false;

	if (_tcsicmp(pControl->GetClass(), _T("ListContainerElementUI")) != 0)
		return false;

	if (reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()) == NULL)
		return CListUI::RemoveAt(iIndex);
	else
		return RemoveNode(reinterpret_cast<Node*>(static_cast<CListContainerElementUI*>(pControl->GetInterface(_T("ListContainerElement")))->GetTag()));
}

void COrganizationTreeUI::RemoveAll()
{
	CListUI::RemoveAll();
	int nCount = root_node_->num_children();
	for (int i = nCount - 1; i >=0;  i--)
	{
		Node* child = root_node_->child(i);
		RemoveNode(child);
	}
	delete root_node_;

	root_node_ = new Node;
	root_node_->data().level_ = -1;
	root_node_->data().child_visible_ = true;
	root_node_->data().has_child_ = true;
	root_node_->data().list_elment_ = NULL;
}

void COrganizationTreeUI::DoEvent(TEventUI& event) 
{
	if (!IsMouseEnabled() && event.Type > UIEVENT__MOUSEBEGIN && event.Type < UIEVENT__MOUSEEND)
	{
		if (m_pParent != NULL)
			m_pParent->DoEvent(event);
		else
			CVerticalLayoutUI::DoEvent(event);
		return;
	}

	if (event.Type == UIEVENT_TIMER && event.wParam == SCROLL_TIMERID)
	{
		if (delay_left_ > 0)
		{
			--delay_left_;
			SIZE sz = GetScrollPos();
			LONG lDeltaY =  (LONG)(CalculateDelay((double)delay_left_ / delay_number_) * delay_deltaY_);
			if ((lDeltaY > 0 && sz.cy != 0)  || (lDeltaY < 0 && sz.cy != GetScrollRange().cy ))
			{
				sz.cy -= lDeltaY;
				SetScrollPos(sz);
				return;
			}
		}
		delay_deltaY_ = 0;
		delay_number_ = 0;
		delay_left_ = 0;
		m_pManager->KillTimer(this, SCROLL_TIMERID);
		return;
	}
	if (event.Type == UIEVENT_SCROLLWHEEL)
	{
		LONG lDeltaY = 0;
		if (delay_number_ > 0)
			lDeltaY =  (LONG)(CalculateDelay((double)delay_left_ / delay_number_) * delay_deltaY_);
		switch (LOWORD(event.wParam))
		{
		case SB_LINEUP:
			if (delay_deltaY_ >= 0)
				delay_deltaY_ = lDeltaY + 8;
			else
				delay_deltaY_ = lDeltaY + 12;
			break;
		case SB_LINEDOWN:
			if (delay_deltaY_ <= 0)
				delay_deltaY_ = lDeltaY - 8;
			else
				delay_deltaY_ = lDeltaY - 12;
			break;
		}
		if
			(delay_deltaY_ > 100) delay_deltaY_ = 100;
		else if
			(delay_deltaY_ < -100) delay_deltaY_ = -100;

		delay_number_ = (DWORD)sqrt((double)abs(delay_deltaY_)) * 5;
		delay_left_ = delay_number_;
		m_pManager->SetTimer(this, SCROLL_TIMERID, 50U);
		return;
	}

	CListUI::DoEvent(event);
}

Node* COrganizationTreeUI::GetRoot()
{
	return root_node_;
}

const TCHAR* const kLogoButtonControlName = _T("logo");
const TCHAR* const kLogoContainerControlName = _T("logo_container");
const TCHAR* const kNickNameControlName = _T("nickname");
const TCHAR* const kDescriptionControlName = _T("description");
const TCHAR* const kOperatorPannelControlName = _T("operation");

static bool OnLogoButtonEvent(void* event) 
{
	TEventUI *pEvent = (TEventUI*)event;
	static CString strOldImage = _T("");
    if( pEvent->Type == UIEVENT_BUTTONDOWN ) 
	{
		CControlUI* pButton = pEvent->pSender;
		if( pButton != NULL ) 
		{
			CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
			if( pListElement != NULL ) 
			{
				COrganizationTreeUI *pThis = (COrganizationTreeUI *)pListElement->GetOwner();
				CButtonUI* logo_hot_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("logo_hot")));
				CButtonUI* logo_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
				if (logo_hot_button!=NULL && logo_button!=NULL)
				{
					//CRect rcpadding(0,0,0,0);
					//pButton->SetPadding(rcpadding);
					//logo_hot_button->SetVisible(true);
					CRect rcItem = pButton->GetPos();
					Node* pNode = (Node*)pListElement->GetTag();
					if (pNode != NULL)
					{
						RECT rcClient = {0};	
						GetWindowRect(GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd(), &rcClient);			
						RECT rcAround = {rcClient.left + rcItem.left - 5, rcClient.top + rcItem.top, rcClient.left + rcItem.right + 5, rcClient.top + rcItem.bottom};//organization_item.xml
						if (GET_CONFIG()->EnableFunction(_T("UI_03")))
						{
							GET_MESSENGER_DLG()->GetProfileDlgController()->CreateProfileDlg(pNode->data().value.GetData(), rcAround, GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd());
						}
					}

					CString strImage = _T("img_main_mouse_move.png");
					logo_button->SetHotImage(strImage);
					logo_button->SetNormalImage(strImage);
				}
				pListElement->DoEvent(*pEvent);
			}
		}
    }
	else if( pEvent->Type == UIEVENT_MOUSEENTER )
	{
		CControlUI* pButton = pEvent->pSender;
		if( pButton != NULL ) 
		{
			CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
			if( pListElement != NULL ) 
			{
				COrganizationTreeUI *pThis = (COrganizationTreeUI *)pListElement->GetOwner();
				CButtonUI* logo_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
				if ( logo_button!=NULL)
				{
					CRect rcpadding(0,0,0,0);
					pButton->SetPadding(rcpadding);
					//logo_hot_button->SetVisible(true);
					CRect rcItem = pButton->GetPos();
					Node* pNode = (Node*)pListElement->GetTag();
					CString strImage = /*GetAppDUIImgDir() +*/ _T("\\img_main_mouse_move.png");
					strOldImage = logo_button->GetNormalImage();
					logo_button->SetNormalImage(strImage);
					logo_button->SetHotImage(strImage);
				}
				pListElement->DoEvent(*pEvent);
			}
		}
		/* 
		CControlUI* pButton = pEvent->pSender;
		if( pButton != NULL ) 
		{
			CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
			if( pListElement != NULL ) 
			{
				COrganizationTreeUI *pThis = (COrganizationTreeUI *)pListElement->GetOwner();
				CButtonUI* log_hot_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, _T("logo_hot")));
				if (log_hot_button != NULL)
				{
					CRect rcpadding(0,0,0,0);
					pButton->SetPadding(rcpadding);
					log_hot_button->SetVisible(true);
					CRect rcItem = pButton->GetPos();
					Node* pNode = (Node*)pListElement->GetTag();
					if (pNode != NULL)
					{
						pThis->ShowProfileWnd(rcItem.TopLeft(), pNode->data().value);
					}
				}
				pListElement->DoEvent(*pEvent);
			}
		}
		*/
	}
	else if( pEvent->Type == UIEVENT_MOUSELEAVE )
	{
		CControlUI* pButton = pEvent->pSender;
        if( pButton != NULL ) 
		{
            CListContainerElementUI* pListElement = (CListContainerElementUI*)(pButton->GetTag());
            if( pListElement != NULL ) 
			{
				COrganizationTreeUI *pThis = (COrganizationTreeUI *)pListElement->GetOwner();
				CButtonUI* logo_button = static_cast<CButtonUI*>(pThis->paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
				if ( logo_button!=NULL)
				{
					//CRect rcpadding(5,0,0,0);
					//pButton->SetPadding(rcpadding);
					//log_hot_button->SetVisible(false);
					//logo_button->SetBkImage(strOldImage);
					logo_button->SetNormalImage(strOldImage);
					logo_button->SetHotImage(strOldImage);
				}
				pListElement->DoEvent(*pEvent);
			}
        }
	}
    return true;
}

void COrganizationTreeUI::RequestUserImage(LPCTSTR strID)
{
	
}

Node* COrganizationTreeUI::AddNode(const OrganizationListItemInfo& item, Node* parent, int nTopPadding)
{
	if (!parent)
	{
		parent = root_node_;
	}

	TCHAR szBuf[MAX_PATH] = {0};

    CListContainerElementUI* pListElement = NULL;
    if( !m_dlgBuilder.GetMarkup()->IsValid() ) 
	{
        pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create(_T("organization_item.xml"), (UINT)0, NULL, &paint_manager_));
    }
    else 
	{
        pListElement = static_cast<CListContainerElementUI*>(m_dlgBuilder.Create((UINT)0, &paint_manager_));
    }
    if (pListElement == NULL)
        return NULL;

	Node* node = new Node;

	node->data().level_ = parent->data().level_ + 1;
	if (item.folder)
	{
		node->data().has_child_ = !item.empty;
	}
	else
	{
		node->data().has_child_ = false;
	}

	node->data().folder_ = item.folder;

	node->data().child_visible_ = (node->data().level_ == 0);
	node->data().child_visible_ = false;

	node->data().text_ = item.nick_name;
	node->data().value = item.id;
	node->data().list_elment_ = pListElement;

	if (!parent->data().child_visible_)
		pListElement->SetVisible(false);

	if (parent != root_node_ && !parent->data().list_elment_->IsVisible())
		pListElement->SetVisible(false);

	CDuiRect rcPadding = text_padding_;
	
	rcPadding.top += nTopPadding;
	for (int i = 0; i < node->data().level_; ++i)
	{
		rcPadding.left += 24;
	}
	if (!node->folder())
	{
		rcPadding.left += 5;
	}
	pListElement->SetPadding(rcPadding);

	CButtonUI* log_button = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoButtonControlName));
	if (log_button != NULL)
	{
		if (!item.folder && !item.logo.IsEmpty())
		{
			CButtonUI* log_hot_button = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(pListElement, _T("logo_hot")));
			if (log_hot_button != NULL)
			{
				log_hot_button->SetVisible(false);
			}
			_stprintf_s(szBuf, MAX_PATH - 1, _T("%s"), item.logo);
			log_button->SetBkImage(szBuf);
			
			
		}
		else
		{
			CContainerUI* logo_container = static_cast<CContainerUI*>(paint_manager_.FindSubControlByName(pListElement, kLogoContainerControlName));
			if (logo_container != NULL)
				logo_container->SetVisible(false);
		}
        log_button->SetTag((UINT_PTR)pListElement);
        log_button->OnEvent += MakeDelegate(&OnLogoButtonEvent);
	}

	CHorizontalLayoutUI* pMobileOnline = static_cast<CHorizontalLayoutUI*>(paint_manager_.FindSubControlByName(pListElement, kOrgMobileOnline));
	CButtonUI* pBtnIPT = static_cast<CButtonUI*>(paint_manager_.FindSubControlByName(pListElement, kOrgBtnIPT));
	CLabelUI* pFavorite = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kFavorite));

	if (pMobileOnline!=NULL && pBtnIPT!=NULL && pFavorite!=NULL)
	{
		if (node->folder())
		{
			pFavorite->SetVisible(false);
		}
		else
		{
			pFavorite->SetVisible(true);
			pFavorite->SetTag((UINT_PTR)node);
			pBtnIPT->SetTag((UINT_PTR)node);
		}	
		if(item.id.CompareNoCase(g_pMessengerActor->GetUserID()) == 0)
		{
			pFavorite->SetEnabled(false);
			pFavorite->SetToolTip(LoadStringFromResource(IDS_STRING_NOT_ADD_SELF));
		}
	}

	CDuiString html_text;
	if (node->data().has_child_)
	{
		if (node->data().child_visible_)
			html_text += level_expand_image_;
		else
			html_text += level_collapse_image_;
		_stprintf_s(szBuf, MAX_PATH - 1, _T("<x %d>"), 8);
		html_text += szBuf;
	}

	CString strBuf = _T("");
	if (item.folder)
	{
		strBuf.Format(_T("<c #666666><f 0>%s</f></c>"), node->data().text_);
		html_text += strBuf;
	}
	else
	{
		_stprintf_s(szBuf, MAX_PATH - 1, _T("<x 6><c #A9A9A9><f 1>%s</f></c>"), item.description);
		strBuf.Format(_T("<c #000000><f 0>%s</f></c>"), item.nick_name);
		strBuf += szBuf;
		html_text += strBuf;
	}

	CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kNickNameControlName));
	if (nick_name != NULL)
	{
		nick_name->SetShowHtml(true);
		nick_name->SetText(html_text);
		strBuf = item.nick_name;
		strBuf += _T("  ");
		strBuf += item.description;
		nick_name->SetToolTip(strBuf);
	}
/*
	if (!item.folder && !item.description.IsEmpty())
	{
		CLabelUI* description = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(pListElement, kDescriptionControlName));
		if (description != NULL)
		{
			_stprintf_s(szBuf, MAX_PATH - 1, _T("<x 0><c #A9A9A9>%s</c>"), item.description);
			description->SetShowHtml(true);
			description->SetText(item.description);
		}
	}
*/	
	if (item.folder)
	{
		pListElement->SetFixedHeight(kOrganizationItemFileHeight);
		CRect rcFile(0, 1, 0, 1);
		pListElement->SetSelectPadding(rcFile);
	}
	else
	{
		pListElement->SetFixedHeight(kOrganizationItemUserHeight);
		CRect rcUser(0, 4, 0, 4);
		pListElement->SetSelectPadding(rcUser);
	}
	
	pListElement->SetTag((UINT_PTR)node);
	int index = 0;
	if (parent->has_children())
	{
		Node* prev = parent->get_last_child();
		index = prev->data().list_elment_->GetIndex() + 1;
	}
	else 
	{
		if (parent == root_node_)
			index = 0;
		else
			index = parent->data().list_elment_->GetIndex() + 1;
	}
	if (!CListUI::AddAt(pListElement, index))
	{
		delete pListElement;
		delete node;
		node = NULL;
	}

	parent->add_child(node);
	return node;
}

bool COrganizationTreeUI::RemoveNode(Node* node)
{
	if (!node || node == root_node_) return false;

	for (int i = 0; i < node->num_children(); ++i)
	{
		Node* child = node->child(i);
		RemoveNode(child);
	}

	CListUI::Remove(node->data().list_elment_);
	node->parent()->remove_child(node);
	delete node;

	return true;
}

void COrganizationTreeUI::SetChildVisible(Node* node, bool visible)
{
	if (!node || node == root_node_)
		return;

	if (node->data().child_visible_ == visible)
		return;

	node->data().child_visible_ = visible;

	CDuiString html_text;
	if (node->data().has_child_)
	{
		if (node->data().child_visible_)
			html_text += level_expand_image_;
		else
			html_text += level_collapse_image_;

		CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));
		if (nick_name != NULL)
		{
			CString strBuf = _T("");
			strBuf.Format(_T("<x 8><c #666666><f 0>%s</f></c>"), node->data().text_);
			html_text += strBuf;
			nick_name->SetShowHtml(true);
			nick_name->SetText(html_text);
		}
	}

	if (!node->data().list_elment_->IsVisible())
		return;

	if (!node->has_children())
		return;

	Node* begin = node->child(0);
	Node* end = node->get_last_child();
	for (int i = begin->data().list_elment_->GetIndex(); i <= end->data().list_elment_->GetIndex(); ++i)
	{
		CControlUI* control = GetItemAt(i);
		if (_tcsicmp(control->GetClass(), _T("ListContainerElementUI")) == 0)
		{
			if (visible) 
			{
				Node* local_parent = ((Node*)control->GetTag())->parent();
				if (local_parent->data().child_visible_ && local_parent->data().list_elment_->IsVisible())
				{
					control->SetVisible(true);
				}
			}
			else
			{
				control->SetVisible(false);
			}
		}
	}
}

void COrganizationTreeUI::ExpandNode(Node* node, bool visible)
{
	node->data().child_visible_ = visible;

	CDuiString html_text;
	if (node->data().has_child_)
	{
		if (node->data().child_visible_)
			html_text += level_expand_image_;
		else
			html_text += level_collapse_image_;

		CLabelUI* nick_name = static_cast<CLabelUI*>(paint_manager_.FindSubControlByName(node->data().list_elment_, kNickNameControlName));
		if (nick_name != NULL)
		{
			CString strBuf = _T("");
			strBuf.Format(_T("<x 8><c #666666><f 0>%s</f></c>"), node->data().text_);
			html_text += strBuf;
			nick_name->SetShowHtml(true);
			nick_name->SetText(html_text);
		}
	}

	node->data().list_elment_->SetVisible(visible);
	
	if (!node->has_children())
		return;

	Node* begin = node->child(0);
	Node* end = node->get_last_child();
	for (int i = begin->data().list_elment_->GetIndex(); i <= end->data().list_elment_->GetIndex(); ++i)
	{
		CControlUI* control = GetItemAt(i);
		if (_tcsicmp(control->GetClass(), _T("ListContainerElementUI")) == 0)
		{
			if (visible) 
			{
				Node* local_parent = ((Node*)control->GetTag())->parent();
				if (local_parent->data().child_visible_ && local_parent->data().list_elment_->IsVisible())
				{
					control->SetVisible(true);
				}
			}
			else
			{
				control->SetVisible(false);
			}
		}
	}
}

bool COrganizationTreeUI::CanExpand(Node* node) const
{
	if (!node || node == root_node_)
		return false;

	return node->data().has_child_;
}

void COrganizationTreeUI::OnMouseButtonEvent(int nTypeEvent, int nIndex)
{
	if (nTypeEvent == UIEVENT_RBUTTONDOWN)
	{

		//SelectItem(nIndex);
	}
	else if (nTypeEvent == UIEVENT_RBUTTONUP)
	{

	}
}



/*bool COrganizationTreeUI::SelectItem( int iIndex, bool bTakeFocus)
{
	bool bRet = false;
	if(0x8000 & GetKeyState(VK_CONTROL))
	{
		bRet = SelectItemCtrl(iIndex, bTakeFocus);
	}
	else if (0x8000 & GetKeyState(VK_SHIFT))
	{
		bRet = SelectItemShift(iIndex, bTakeFocus);
	}
	else
	{
		bRet = SelectItemNormal(iIndex, bTakeFocus);
	}

	return bRet;
}

bool COrganizationTreeUI::SetSelectItem(int iIndex, bool bTakeFocus)
{
	IListItemUI* pListItem = GetListItemUI(iIndex);
	if (pListItem!=NULL && !pListItem->IsSelected())
	{
		if (!__super::SelectItem(iIndex, bTakeFocus))
			return false;
	}
	return true;
}*/

IListItemUI* COrganizationTreeUI::GetListItemUI(int nIndex)
{
	IListItemUI* pListItem = NULL;
	CControlUI* pControl = GetItemAt(nIndex);
	if( pControl != NULL) 
	{
		pListItem = static_cast<IListItemUI*>(pControl->GetInterface(_T("ListItem")));
	}
	return pListItem;
}

Node* COrganizationTreeUI::GetItemNode(int nIndex)
{
	Node *pNode = NULL;
	CControlUI* pControl = GetItemAt(nIndex);
	if(pControl != NULL)
	{
		pNode = (Node*)pControl->GetTag();
	}
	return pNode;
}

/*bool COrganizationTreeUI::SelectItemCtrl(int iIndex, bool bTakeFocus)
{
	if( iIndex < 0 )
	{
		return false;
	}
	POSITION pos = m_listSelRow.Find(iIndex);
	if (pos == NULL)
	{
		m_iCurSel = -1;
		if (!SetSelectItem(iIndex, bTakeFocus))
		{
			return false;
		}
		m_listSelRow.AddTail(iIndex);
	}
	else
	{
		IListItemUI* pListItem = GetListItemUI(iIndex);
		if( pListItem != NULL )
		{
			pListItem->Select(false);
		}

		m_listSelRow.RemoveAt(pos);
	}
	
	return true;
}

bool COrganizationTreeUI::SelectItemShift(int iIndex, bool bTakeFocus)
{
	static bool s_bStartCycle = false;
	if (s_bStartCycle)
	{
		return true;
	}
	POSITION pos = m_listSelRow.GetHeadPosition();
	while(pos != NULL)
	{
		int nIndex = m_listSelRow.GetNext(pos);
		if (nIndex >= 0)
		{
			IListItemUI* pListItem = GetListItemUI(nIndex);
			if( pListItem != NULL )
			{
				pListItem->Select(false);
			}
		}
	}
	m_listSelRow.RemoveAll();

	if((m_nShiftFirstSel>=0) && (iIndex>=0))
	{
		s_bStartCycle = true;
		if(iIndex > m_nShiftFirstSel)
		{
			for (int i=m_nShiftFirstSel; i<iIndex+1; ++i)
			{
				m_iCurSel = -1;
				if (!SetSelectItem(i, bTakeFocus))
				{
					return false;
				}
				m_listSelRow.AddTail(i);
			}
		}
		else if(iIndex < m_nShiftFirstSel)
		{
			for (int i=iIndex; i<m_nShiftFirstSel+1; ++i)
			{
				m_iCurSel = -1;
				if (!SetSelectItem(i, bTakeFocus))
				{
					return false;
				}
				m_listSelRow.AddTail(i);
			}
		}
	}

	if (m_nShiftFirstSel < 0)
	{
		SelectItemNormal(iIndex, bTakeFocus);
	}
	s_bStartCycle = false;
	
	return true;
}

bool COrganizationTreeUI::SelectItemNormal(int iIndex, bool bTakeFocus)
{
	m_nShiftFirstSel = iIndex;
	POSITION pos = m_listSelRow.GetHeadPosition();
	while(pos != NULL)
	{
		int nIndex = m_listSelRow.GetNext(pos);
		if (nIndex >= 0)
		{
			IListItemUI* pListItem = GetListItemUI(nIndex);
			if( pListItem != NULL )
			{
				pListItem->Select(false);
			}
		}
	}
	m_listSelRow.RemoveAll();
	if((m_iCurSel>=0) && (m_iCurSel!=iIndex)) 
	{
		IListItemUI* pListItem = GetListItemUI(m_iCurSel);
		if( pListItem != NULL )
		{
			pListItem->Select(false);
		}
		m_iCurSel = -1;
	}
	if( iIndex < 0 )
		return false;
	if (!SetSelectItem(iIndex, bTakeFocus))
		return false;
	m_listSelRow.AddTail(iIndex);
	return true;
}*/

} // namespace DuiLib
